import { BackendId, UniqueBackendIdentifier } from '@aws-amplify/plugin-types';
/**
 * UniqueBackendIdentifierBase
 */
export declare abstract class UniqueBackendIdentifierBase implements UniqueBackendIdentifier {
    /**
     * For Amplify branch environments, this is the Amplify app id
     * For sandbox deployments, this is a concatenation of package.json#name and the current local username
     */
    readonly backendId: BackendId;
    /**
     * Disambiguator for the identifier
     */
    readonly disambiguator: string;
    /**
     * UniqueBackendIdentifierBase
     */
    constructor(
    /**
     * For Amplify branch environments, this is the Amplify app id
     * For sandbox deployments, this is a concatenation of package.json#name and the current local username
     */
    backendId: BackendId, 
    /**
     * Disambiguator for the identifier
     */
    disambiguator: string);
}
/**
 * BranchBackendIdentifier
 */
export declare class BranchBackendIdentifier extends UniqueBackendIdentifierBase {
    readonly backendId: BackendId;
    /**
     * BranchBackendIdentifier
     */
    constructor(backendId: BackendId, branchName: string);
}
/**
 * SandboxBackendIdentifier
 */
export declare class SandboxBackendIdentifier extends UniqueBackendIdentifierBase {
    readonly backendId: BackendId;
    /**
     * SandboxBackendIdentifier
     */
    constructor(backendId: BackendId);
    /**
     * Parses identifier instance from sandbox name
     */
    static tryParse(sandboxName: string): SandboxBackendIdentifier | undefined;
}
//# sourceMappingURL=unique_backend_identifier.d.ts.map